@@ -7,9 +7,9 @@
 #
 # Authors:
 # - Ralph Vigne, <ralph.vigne@cern.ch>, 2013 - 2014
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2012
+# - Mario Lassnig, <mario.lassnig@cern.ch>, 2012, 2014
 # - Vincent Garonne, <vincent.garonne@cern.ch>, 2013
-# - Cedric Serfon, <cedric.serfon@cern.ch>, 2013
+# - Cedric Serfon, <cedric.serfon@cern.ch>, 2013-2014
 
 import copy
 import os
@@ -18,7 +18,7 @@ from dogpile.cache import make_region
 from urlparse import urlparse
 
 from rucio.common import exception, utils
-
+from rucio.db.session import read_session
 
 DEFAULT_PROTOCOL = 1
 
@@ -31,6 +31,7 @@ region = make_region(function_key_generator=utils.my_key_generator).configure(
 
 
 @region.cache_on_arguments(namespace='rse_info')  # Sets the dogpile.cache for this method based on the provided RSE name
+@read_session
 def get_rse_info(rse, session=None):
     """ Returns all protocol related RSE attributes.
 
@@ -88,7 +89,7 @@ def select_protocol(rse_settings, operation, scheme=None):
         candidates.remove(r)
 
     if not len(candidates):
-        raise exception.RSEProtocolNotSupported('No protocol for provided settings found.' % rse_settings['domain'])
+        raise exception.RSEProtocolNotSupported('No protocol for provided settings found : %s.' % str(rse_settings))
 
     # Select the one with the highest priority
     best_choice = candidates[0]
@@ -172,7 +173,7 @@ def parse_pfns(rse_settings, pfns, operation='read'):
     return create_protocol(rse_settings, operation, urlparse(pfns[0]).scheme).parse_pfns(pfns)
 
 
-def download(rse_settings, files, dest_dir='.'):
+def download(rse_settings, files, dest_dir='.', printstatements=False):
     """
         Copy a file from the connected storage to the local file system.
         Providing a list indicates the bulk mode.
@@ -189,6 +190,7 @@ def download(rse_settings, files, dest_dir='.'):
 
         :raises SourceNotFound: remote source file can not be found on storage
         :raises DestinationNotAccessible: local destination directory is not accessible
+        :raises FileConsistencyMismatch: the checksum of the downloaded file does not match the provided one
         :raises ServiceUnavailable: for any other reason
 
     """
@@ -204,8 +206,30 @@ def download(rse_settings, files, dest_dir='.'):
             if not os.path.exists('%s/%s' % (dest_dir, f['scope'])):
                 os.makedirs('%s/%s' % (dest_dir, f['scope']))
             # Each scope is stored into a separate folder
-            protocol.get(pfn, '%s/%s/%s' % (dest_dir, f['scope'], f['name']))
-            ret['%s:%s' % (f['scope'], f['name'])] = True
+            finalfile = '%s/%s/%s' % (dest_dir, f['scope'], f['name'])
+            # Check if the file already exists, if not download and validate it
+            if not os.path.isfile(finalfile):
+                if 'adler32' in f:
+                    tempfile = '%s/%s/%s.part' % (dest_dir, f['scope'], f['name'])
+                    if os.path.isfile(tempfile):
+                        if printstatements:
+                            print '%s already exists, probably from a failed attempt. Will remove it' % (tempfile)
+                        os.unlink(tempfile)
+                    protocol.get(pfn, tempfile)
+                    if printstatements:
+                        print 'File downloaded. Will be validated'
+                    localchecksum = utils.adler32(tempfile)
+                    if localchecksum == f['adler32']:
+                        if printstatements:
+                            print 'File validated'
+                        os.rename(tempfile, finalfile)
+                    else:
+                        raise exception.FileConsistencyMismatch('Checksum mismatch : local %s vs recorded %s' % (str(localchecksum), str(f['adler32'])))
+                else:
+                    protocol.get(pfn, '%s/%s/%s' % (dest_dir, f['scope'], f['name']))
+                ret['%s:%s' % (f['scope'], f['name'])] = True
+            else:
+                ret['%s:%s' % (f['scope'], f['name'])] = True
         except Exception as e:
             gs = False
             ret['%s:%s' % (f['scope'], f['name'])] = e
